//========================================================================================
//
//	File:				DrawSel.h
//	Release Version:	$ ODF 1 $
//
//	Author:				Henri Lamiraux
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef DRAWSEL_H
#define DRAWSEL_H

// ----- Part Layer -----

#ifndef FWSELECT_H
#include "FWSelect.h"
#endif

// ----- OS Layer -----

#ifndef FWRECT_H
#include "FWRect.h"
#endif

#ifndef FWRECSHP_H
#include "FWRecShp.h"
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class FW_CGraphicContext;
class FW_CMouseEvent;

class CDrawPart;
class CDrawFrame;
class CSelectionIterator;
class CDrawSelection;
class CBaseShape;

class CDrawContent;
class CDrawSelectionContent;

//========================================================================================
//	class CDrawSelection
//========================================================================================

class CDrawSelection : public FW_CSelection
{
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:
	FW_DECLARE_AUTO(CDrawSelection)

	CDrawSelection(Environment* ev, CDrawPart* drawPart);
	virtual 		~CDrawSelection();
	
//----------------------------------------------------------------------------------------
//	Inherited API
//
public:
	virtual void		CloseSelection(Environment* ev);
	virtual void		SelectAll(Environment* ev);
	virtual FW_Boolean	IsEmpty(Environment* ev) const;
	virtual void		ClearSelection(Environment* ev);
	
	virtual FW_CContent* GetSelectedContent(Environment* ev);
	
	virtual ODShape*	CreateSelectionShape(Environment* ev, ODFacet* facet, FW_CFrame* frame) const;
	virtual ODShape*	CreateSelectionOutline(Environment* ev, ODFacet* facet, FW_CFrame* frame) const;
	
	virtual FW_Boolean	CanPasteAsLink(Environment* ev, ODPasteAsMergeSetting& setting, ODStorageUnit* su);
	virtual FW_Boolean	IsSelectionLinkable(Environment* ev);
	
	virtual FW_Boolean	IsMouseInDraggableItem(Environment* ev, 
							FW_CFrame* frame,
							const FW_CMouseEvent& theMouseEvent, 
							FW_Boolean inBackground);

	virtual void		UpdateSelectionOnMouseDown(Environment* ev, 
							const FW_CMouseEvent& mouseEvent,
							ODFacet* embeddedFacet,
							FW_Boolean inEmbeddedFrameBorder,
							FW_Boolean inBackground);
	
//----------------------------------------------------------------------------------------
//	New API
//
public:
	CBaseShape* 	WhichHandle(Environment* ev, FW_CGraphicContext& gc, const FW_CPoint& mouse, short& whichHandle) const;
	
	void			RenderSelectionHandles(Environment* ev, FW_CGraphicContext& gc);
	void			RenderHandles(Environment* ev, CBaseShape* shape);
	void			RenderAllHandles(Environment* ev, FW_CFrame* frame);
	
	void			AddToSelection(Environment* ev, CBaseShape* theShape, FW_Boolean renderHandles);
	void			RemoveFromSelection(Environment* ev, CBaseShape* theShape, FW_Boolean renderHandles);

	void			OffsetSelection(Environment* ev, FW_Fixed xDelta, FW_Fixed yDelta);

	FW_Boolean		Resize(Environment* ev, const FW_CMouseEvent& theMouseEvent);	

	void			SelectWithRectangle(Environment* ev, const FW_CMouseEvent& theMouseEvent);

	FW_Boolean		GetSelectionPenSize(Environment* ev, FW_Fixed& penSize);

	FW_CPoint		CenterSelection(Environment* ev, FW_CFrame* scopeFrame);
	
	unsigned long	Count() const;
	
	// ----- Frozen state -----
	FW_Boolean		HasFrozen() const;
	FW_Boolean		HasNotFrozen() const;
	void			SetFrozen(Environment* ev, FW_Boolean state);
	
	ODShape*		GetUpdateShape() const;
	void			GetDragRect(FW_CRect& dragRect) const;

	// ----- Anchor Shape -----
	CBaseShape*		GetAnchorShape() const;
	void			SetAnchorShape(CBaseShape* shape);


	// ----- Linking -----
	void			SelectionChanged(Environment* ev);
	void			DeleteSelection(Environment* ev);
	FW_Boolean		IsOKtoEdit(Environment* ev);

	void			CalcCache(Environment* ev);

private:
	
	void			DoAdd(Environment* ev, CBaseShape *shape);
	void			DoRemove(Environment* ev, CBaseShape *shape);
		
	FW_Boolean		IsShapeInLinkDestination(Environment* ev);

public:
	void			ShapeAdded(Environment* ev, CBaseShape* shape);
	void			SelectContent(Environment* ev, CDrawContent* content);
	CDrawContent*	GetDrawContent(Environment* ev);

//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	CDrawPart*			fDrawPart;
	FW_CRect			fDragRect;
	ODShape*			fUpdateShape;
	unsigned short		fFrozenCount;
	unsigned long		fCount;
	CBaseShape*			fAnchorShape;
	short				fClickedHandle;
	
	CDrawSelectionContent*	fSelectionContent;
	
	FW_CRectShape		fWorkingHandle;
	CDrawSelectionContent*	fDraggedContent;
};

//========================================================================================
//	CDrawSelection Inlines
//========================================================================================

//----------------------------------------------------------------------------------------
//	CDrawSelection::GetUpdateShape
//----------------------------------------------------------------------------------------
inline ODShape* CDrawSelection::GetUpdateShape() const
{		
	return fUpdateShape;
}

//----------------------------------------------------------------------------------------
//	CDrawSelection::GetDragRect
//----------------------------------------------------------------------------------------
inline void CDrawSelection::GetDragRect(FW_CRect& dragRect) const
{		
	dragRect = fDragRect;
}

//----------------------------------------------------------------------------------------
//	CDrawSelection::HasFrozen
//----------------------------------------------------------------------------------------
inline FW_Boolean CDrawSelection::HasFrozen() const
{
	return fFrozenCount != 0;
}

//----------------------------------------------------------------------------------------
//	CDrawSelection::HasNotFrozen
//----------------------------------------------------------------------------------------
inline FW_Boolean CDrawSelection::HasNotFrozen() const
{
	return fCount != 0 && fFrozenCount != fCount;
}

//----------------------------------------------------------------------------------------
//	CDrawSelection::Count
//----------------------------------------------------------------------------------------
inline unsigned long CDrawSelection::Count() const
{
	return fCount;
}

//----------------------------------------------------------------------------------------
//	CDrawSelection::GetAnchorShape
//----------------------------------------------------------------------------------------
inline CBaseShape* CDrawSelection::GetAnchorShape() const
{
	return fAnchorShape;
}

//----------------------------------------------------------------------------------------
//	CDrawSelection::SetAnchorShape
//----------------------------------------------------------------------------------------
inline void CDrawSelection::SetAnchorShape(CBaseShape* shape)
{
	fAnchorShape = shape;
}

#endif